#include "TimerOne.h" 
#include <EEPROM.h>

const String splashscreen="Diversity by Fisch V20150629";
#define EEPROMVERSION 2

/*
  TODO:
  
  Bugs:

*/

/*
  Changes V20150629
  *added RSSI_DISCONNECTED to avoid switching to a not connected receiver input
  *added receivermode 0-3 to eeprom
  *added receivermode 0-3 entries in debug menu, hold B to cycle trough the modes
  *added MAXSYNCFAILS, so that static is better rated than blackscreen(not connected)

  Changes V0.13 = V20150318
  *atmega pin changes for better pcb layout (PCB Vers 20150310/20150313)
  *Debug Menu entry
  
*/


/*
  defines and variables with "//###########" can be changed
  "                     with "//##########EEPROM" are stored in EEPROM and can be changed in the Diversity Menu
*/

#define PIN_STATUSLED 13
#define PIN_INT_SYNC 0 //interrupt 0, arduino d2 = atmega pin 4
//interrupt 1, arduino d3 = atmega pin 5
#define PIN_RSSI3 A0 //A0 = atmega pin 23
#define PIN_RSSI2 A1 //A1 = atmega pin 24
#define PIN_RSSI1 A2 //A2 = atmega pin 25
#define PIN_RSSI0 A3 //A3 = atmega pin 26
#define PIN_SELECT0 7//d7 = atmega pin 13 (pd7)
#define PIN_SELECT1 6//d6 = atmega pin 12 (pd6)
#define PIN_SELECTSYNC0 10 //D10 = atmega pin 16 (PB2)
#define PIN_SELECTSYNC1 9//D9 = atmega pin 15 (PB1)

#define PIN_BUTTON_A 4 //D4 = atmega pin 6
#define PIN_BUTTON_B 5 //D5 = atmega pin 11
#define BUTTON_HOLDDOWNTIME 750 //Holddown time for button
#define BUTTON_DEBOUNCETIME 150
#define PIN_SPEAKER 8 //D8 = atmega pin 14


#define MAXRX 4 //do not change


//Diversity setting defaults
#define DEFAULT_RSSIUPDATE_ALPHA 80 //smoothing rssi, in percent. lower value=more smoothing
#define DEFAULT_SYNCSELECTIONDIFF 500 //minimum sync difference to select receiver, if difference is lower then rssi will be compared
#define DEFAULT_SWITCHDEADTIME 500 //time (in ms) to wait, after switching 

//serial command stuff
#define BAUD 9600
#define ENTER 13            // byte value of the Enter key (LF)
String serialprint=String();
uint8_t serial_mode=0; //0=off, 1=writing header, 2=writing data, 10=debug
boolean osdconnected=true; //format and delay output to display on osd
boolean serialPrintOnReceiverChange=true; //print values immediately if receiver has been switched

//Variablen
uint8_t mode=0; //0=normal diversity op, 1=manual selection mode
uint8_t setupmenu=0; //0=closed, x=entry x
  uint8_t menu_maxentry=6;
  uint8_t menu_selected=0; //0=not selected, 1=entry selected
  uint8_t menu_debugentry=0;
  uint8_t menu_maxdebugentry=8;
/*
  *0=closed
  *1=rssiupdate_alpha [0,99] _5
  *2=syncselectiondiff [0,16000] _10(<100) _50(<500) _100(>)
  *3=switchdeadtime [0,5000] _10(<100) _50(<1000) _100(>)
  *4=load default
  *5=save
  *5=debug
*/

uint16_t freq=8; //timer frequency in Hz, main frequency to calculate and eventually switch video signals //#############
#define CSVSENDINTERVAL 8 //send csv data every x timer interrups
uint8_t csvsendintervalcounter=CSVSENDINTERVAL;
uint8_t flag_timerIsr=0; //flag for timer interrupt
uint16_t maxrawsynccounting= 65535/freq; //precalculate max. synccounting (see syncinterrupt)

uint8_t flag_checkSignals=0; //flag set, then calculate signals and select receiver

uint16_t rawrssi[4]={
  0,0,0,0}; //raw rssi values, 0-1024
uint16_t rssi[4]; //mapped rssi values, range 0-255
uint16_t rssimin[4]={
  1023,1023,1023,1023}; //min. rssi values (calibration)
uint16_t rssimax[4]={
  0,0,0,0}; //max. rssi values (calibration)
uint16_t cutoff_rawrssi[4]; //temp
long lastupdaterssi = 0;
#define RSSIUPDATETIME 10 //time between rssi readings, in ms //#############
uint8_t rssiupdate_alpha=DEFAULT_RSSIUPDATE_ALPHA; //smoothing rssi, in percent //#############EEPROM
uint8_t rssiupdate_alpha_calib=10; //smoothing during calibration //#############
#define CALIBSTART 500 //time to wait until rssi calibration starts, should be at least 100/rssiupdate_alpha_calib * RSSIUPDATETIME


uint16_t rawsync[4]={
  0,0,0,0}; //raw syncs, counts per sync interrupt
uint16_t sync[4]={
  0,0,0,0}; //syncs per second
uint8_t interrupt_sync_input=0; //wich receiver is muxed to interrupt

//uint16_t signalrating[MAXRX]={0,0,0,0}; //rating for signal quality, calculated by rssi and syncs
int16_t syncrating[MAXRX]={
  0,0,0,0};
uint8_t selectedReceiver=0; //switched receiver video signal

uint8_t lastselectedby=0; //0=manually selected, 1=by syncs, 2=by rssi

uint8_t calibration_mode=1; //0=normal ,1=enable, 2=finding max and min values over CALIBRATIONTIME, 3=save to eeprom, 4=load from eeprom
long calibrationtime_start=0; //for timing
#define CALIBRATIONTIME 4000 //minimum calibrationtime in ms //#############

#define DEACTIVATERSSI 40 //deactivate receiver input if rawrssi is less than x (during calibration)//#############
#define RSSI_DISCONNECTED 10 //maximum raw rssi value, if rawrssi is below the receiver gets rssi set to 0, if connected minimum rssi is 1 (during runtime) //###############
#define MAXSYNCFAILS 15000//cutoff maximum syncfails (if rawsyncfails>1600), so that black screen (not connected) is not better than static, should be less than 16000
//#define DEACTIVATESYNC 25 //deactivate receiver input if syncs are less than x //############# //not in use since 20150318
uint8_t receivermode[4]={
  1,1,1,1}; //receivermodes: 0=deactivated (if no receiver connected, checked by calibration), 1=activated, 2=receiver has no reliable rssi (such as an relay station)
#define MAXRECEIVERMODE 2 //for cycling through in debug menu

uint16_t syncselectiondiff=DEFAULT_SYNCSELECTIONDIFF; //minimum sync difference to select receiver, if difference is lower then rssi will be compared //#############EEPROM
uint16_t switchdeadtime=DEFAULT_SWITCHDEADTIME; //time (in ms) to wait, after switching //#############EEPROM
long lastswitched=0;
uint8_t snd_signallost=0; //beep once on signal loss
//#define SIGNALLOSSRSSI 20//if rssi below, signal ist lost (for sound)
//#define SIGNALGOODRSSI 150 //good rssi (for sound)

//             Button A
uint8_t button_down_A=0; //temp variable for button press
long button_down_starttime_A=0;
uint8_t button_released_A=0; //button has been released
uint8_t button_hold_A=0; //button hold
long button_released_time_A=0;
uint8_t button_holded_A=0; //hold was once set to 1

//             Button B
uint8_t button_down_B=0; //temp variable for button press
long button_down_starttime_B=0;
uint8_t button_released_B=0; //button has been released
uint8_t button_hold_B=0; //button hold
long button_released_time_B=0;
uint8_t button_holded_B=0; //hold was once set to 1


//Beeps
#define SNDF_CALIBLOADED 200 //also startup/diversitymode
#define SNDD_CALIBLOADED 500 //also startup/diversitymode
#define SNDF_CALIBLOADFAILED_1 1000
#define SNDD_CALIBLOADFAILED_1 500
#define SNDDELAY_CALIBLOADFAILED 200
#define SNDF_CALIBLOADFAILED_2 300
#define SNDD_CALIBLOADFAILED_2 2000
#define SNDF_CALIBSAVED 1000
#define SNDD_CALIBSAVED 1000
#define SNDF_CALIBRATION 1000
#define SNDD_CALIBRATION 500

#define SNDF_MENUUP 880
#define SNDD_MENUUP 20
#define SNDF_MENUDOWN 660
#define SNDD_MENUDOWN 20
#define SNDF_MENUOPEN 880
#define SNDD_MENUOPEN 100
#define SNDF_MENUSELECTEDBORDER 220
#define SNDD_MENUSELECTEDBORDER 50

#define SNDF_MANUALMODE 440
#define SNDD_MANUALMODE 50
#define SNDDELAY_MANUALMODE (SNDD_MANUALMODE+50)
#define SNDF_MANUALMODE2 440
#define SNDD_MANUALMODE2 50

#define SNDF_R0 440 //sound frequency receiver 0
#define SNDD_R0 50 //sound duration receiver 0, set to 0 to deacitvate
#define SNDF_R1 660 //" receiver 1
#define SNDD_R1 50 //" receiver 1
#define SNDF_R2 880 
#define SNDD_R2 50 
#define SNDF_R3 1100 
#define SNDD_R3 50 

//#define SNDF_SIGNALLOST 150
//#define SNDD_SIGNALLOST 1000

void setup() {             
  pinMode(PIN_STATUSLED, OUTPUT);
  pinMode(PIN_SELECT0, OUTPUT);
  pinMode(PIN_SELECT1, OUTPUT);
  pinMode(PIN_SELECTSYNC0, OUTPUT);
  pinMode(PIN_SELECTSYNC1, OUTPUT);

  pinMode(PIN_RSSI0, INPUT);
  pinMode(PIN_RSSI1, INPUT);
  pinMode(PIN_RSSI2, INPUT);
  pinMode(PIN_RSSI3, INPUT);
  pinMode(PIN_BUTTON_A, INPUT);
  pinMode(PIN_BUTTON_B, INPUT);


  digitalWrite( PIN_STATUSLED, HIGH);
  selectReceiver(0); //preselect receiver 0
  selectSync(0); //preselct sync counting from 0

    Serial.begin(BAUD);
  Serial.println(splashscreen);

  Timer1.initialize(1000000/freq); // set timer
  Timer1.attachInterrupt( timerIsr ); // attach the service routine here

  attachInterrupt(PIN_INT_SYNC,interrupt_sync,RISING); //sync interrupt

  digitalWrite( PIN_STATUSLED, LOW);


}


void loop() {
  if (millis()>=(lastupdaterssi+RSSIUPDATETIME)){
    lastupdaterssi=millis();
    updateRSSI();  //update RSSI values
  }

  //CALIBRATION
  calibration(); //check and do calibration, if calib. done, this method does nothing

  //Comparing
  if (flag_checkSignals==1){
    calcSignalRating(); //calculate signal rating from missing syncs and rssi -> signalrating[]
    if (millis()>=(lastswitched+switchdeadtime)){
      if (mode==0) //switch receivers only in diversity mode
        compareAndSelect(); //compare rssi and syncs and select best signal
    }
    flag_checkSignals=0; //reset flag
  }

  //Timer Interrupt
  if (flag_timerIsr==1){ //ceck timer flag
    flag_timerIsr=0; //reset flag
    timerIsrcall();
  }



  /*if (digitalRead(PIN_BUTTON)==HIGH){
   if (serial_mode==0 && calibration_mode==0 && serial_mode!=1){ //only start csv output if no serial output active AND no calibration active
   serial_mode=1; //starting csv output
   }
   }*/

  //Button Press ++  A
  button_released_A=0;
  button_hold_A=0;
  if (digitalRead(PIN_BUTTON_A)==HIGH && button_down_A==0 && (millis()-button_released_time_A)>=BUTTON_DEBOUNCETIME){//press
    button_down_A=1;
    button_down_starttime_A=millis();
  }
  else if(digitalRead(PIN_BUTTON_A)==LOW && button_down_A==1){//release
    if ( (millis()-button_down_starttime_A)<BUTTON_HOLDDOWNTIME)
      button_released_A=1; //button is not pressed but was down before

    button_down_A=0;
    button_holded_A=0;
    button_released_time_A=millis();
    flag_timerIsr=1; //update osd menu
  }
  else if(digitalRead(PIN_BUTTON_A)==HIGH && button_down_A==1 && button_holded_A==0 && (millis()-button_down_starttime_A)>=BUTTON_HOLDDOWNTIME ){//hold down
    button_down_A=1;
    button_hold_A=1;
    button_holded_A=1;
  }
  //           +++
  //Button Press ++  B
  button_released_B=0;
  button_hold_B=0;
  if (digitalRead(PIN_BUTTON_B)==HIGH && button_down_B==0 && (millis()-button_released_time_B)>=BUTTON_DEBOUNCETIME){//press
    button_down_B=1;
    button_down_starttime_B=millis();
  }
  else if(digitalRead(PIN_BUTTON_B)==LOW && button_down_B==1){//release
    if ( (millis()-button_down_starttime_B)<BUTTON_HOLDDOWNTIME)
      button_released_B=1; //button is not pressed but was down before

    button_down_B=0;
    button_holded_B=0;
    button_released_time_B=millis();
    flag_timerIsr=1; //update osd menu
  }
  else if(digitalRead(PIN_BUTTON_B)==HIGH && button_down_B==1 && button_holded_B==0 && (millis()-button_down_starttime_B)>=BUTTON_HOLDDOWNTIME ){//hold down
    button_down_B=1;
    button_hold_B=1;
    button_holded_B=1;
  }
  //           +++

  //Button handling A <<<<<<<<<<<<<<<<<<<<<<<< KLICK
  
  if (button_released_A==1){ //button pressed (on release)
    if(setupmenu==0) //if menu closed
    {
      
      if(mode!=0){
        mode=0; //switch to diversity mode
        snd_entereddiversitymode();
      }
      
    }else{ //setup menu opened
      if(menu_selected==0){ //in entry selection
        if (setupmenu==1){
          setupmenu=menu_maxentry;
          snd_menuopen();
        }else{
          setupmenu--;
          snd_menudown();
        }
      }else if(menu_selected==1){ //entry selected
          if(setupmenu==1){ 
           rssiupdate_alpha-=5;
          }else if(setupmenu==2){ 
           if (syncselectiondiff<=100){ syncselectiondiff-=10;
           }else if (syncselectiondiff<=500){  syncselectiondiff-=50;
           }else{  syncselectiondiff-=100;} 
          }else if(setupmenu==3){ 
           if (switchdeadtime<=100){ switchdeadtime-=10;
           }else if (switchdeadtime<=1000){ switchdeadtime-=50;
           }else{ switchdeadtime-=100;}
           
          }else if(setupmenu==4){  //load defaults
           //do nothing
          }else if(setupmenu==5){  //save
           //do nothing
          }else if(setupmenu==6){ //debug
           //prev debug entry
           if (menu_debugentry<=0){
             menu_debugentry=menu_maxdebugentry;
           }else{
             menu_debugentry-=1;
           }
             
          }
        snd_menudown();
        
      }
    }
  }
  //>>>>>>>>>>
  if (button_hold_A==1){ //button hold down for BUTTON_HOLDDOWNTIME
    if (setupmenu==0){ //if menu closed
      //nothing
      
    }else{ //if opened
      if (menu_selected==0){ //in menu
        setupmenu=0; //close
        snd_menuclosed();
        
      }else{ //entry selected
        menu_selected=0; //back to entry selection
        snd_menudeselected();
        
      }
    }
  }
  
  //Button B <<<<<<<<<<<<<<<<<<<<<< KLICK
  
  if (button_released_B==1){ //button pressed (on release)
    if(setupmenu==0) //if menu closed
    {
      
      if (mode!=1){
        mode=1; //switch receivers manually
        snd_enteredmanualmode();
        
      }
      if (mode==1){ //pressed in manual mode
        nextReceiver_manual(); //switch receivers man
        
      }
      
    }else{ //setup menu opened
      if(menu_selected==0){ //in entry selection
        if (setupmenu==menu_maxentry){
          setupmenu=1;
          snd_menuopen();
          
        }else{
          setupmenu++;
          snd_menuup();
          
        }
      }else if(menu_selected==1){ //entry selected
        if(setupmenu==1){
           rssiupdate_alpha+=5;
        }else if(setupmenu==2){ 
           if (syncselectiondiff<100){  syncselectiondiff+=10;
           }else if (syncselectiondiff<500){  syncselectiondiff+=50;
           }else{  syncselectiondiff+=100;}
        }else if(setupmenu==3){
           if (switchdeadtime<100){ switchdeadtime+=10;
           }else if (switchdeadtime<1000){ switchdeadtime+=50;
           }else{ switchdeadtime+=100;}
        }else if(setupmenu==4){
           //do nothing, to load default hold down button
        }else if(setupmenu==5){ //save
           //do nothing
        }else if(setupmenu==6){ //debug
           //next debug entry
           menu_debugentry+=1;
           if (menu_debugentry>menu_maxdebugentry)
             menu_debugentry=0;
        }
        snd_menuup();
        
      }
    }
  }
  //>>>>>>>>>> HOLD B
  if (button_hold_B==1){ //button hold down for BUTTON_HOLDDOWNTIME
    if(setupmenu==0){ //if menu closed
      setupmenu=1; //open menu
      
    }else{ //setup menu opened
      if (menu_selected==0){
        menu_selected=1; //select entry
        snd_menuselected();
        
      }else if(menu_selected==1){ //
        if (setupmenu==4){ //load defaults, entry selected and held down
          loadDiversitySettingDefaults(); //load defaults
          snd_loadeddefaults();
          menu_selected=0; //go back
        }else if (setupmenu==5){ //write settings to eeprom
          writeEEPROMsettings();
          snd_eepromwritten();
          menu_selected=0; //go back
        }else if (setupmenu==6){ //debug menu
          if (menu_debugentry==5){ //receivermode entry, hold B to change mode
            receivermode[0]=receivermode[0]++; //cycle through
            if (receivermode[0]>MAXRECEIVERMODE)
              receivermode[0]=0;
          }else if (menu_debugentry==6){
            receivermode[1]=receivermode[1]++; //cycle through
            if (receivermode[1]>MAXRECEIVERMODE)
              receivermode[1]=0;
          }else if (menu_debugentry==7){
            receivermode[2]=receivermode[2]++; //cycle through
            if (receivermode[2]>MAXRECEIVERMODE)
              receivermode[2]=0;
          }else if (menu_debugentry==8){
            receivermode[3]=receivermode[3]++; //cycle through
            if (receivermode[3]>MAXRECEIVERMODE)
              receivermode[3]=0;
          }
        }
      }
      
    }
    
  }
  
  
  // <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  
  
  
  //Borders for Variables
  if(rssiupdate_alpha<0 || rssiupdate_alpha>=150){ //undershoot (<0 not possible)
    rssiupdate_alpha=0;
    snd_menuselectedborder();
  }else if(rssiupdate_alpha>95){
    rssiupdate_alpha=95;
    snd_menuselectedborder();
  }
  if(syncselectiondiff<0 || syncselectiondiff>=65000){
    syncselectiondiff=0;
    snd_menuselectedborder();
  }else if(syncselectiondiff>16000){
    syncselectiondiff=16000;
    snd_menuselectedborder();
  }
  if(switchdeadtime<0 || switchdeadtime>=60000){
    switchdeadtime=0;
    snd_menuselectedborder();
  }else if(switchdeadtime>5000){
    switchdeadtime=5000;
    snd_menuselectedborder();
  }
  


}

void updateRSSI() //read, calculate and save rssi values
{
  uint8_t alpha=rssiupdate_alpha;
  if (calibration_mode==2)
    alpha=rssiupdate_alpha_calib;

  if (receivermode[0]!=0)
    rawrssi[0] = rawrssi[0]*(100-alpha)/100 + analogRead(PIN_RSSI0)*(alpha)/100;
  if (receivermode[1]!=0)
    rawrssi[1] = rawrssi[1]*(100-alpha)/100 + analogRead(PIN_RSSI1)*(alpha)/100;
  if (receivermode[2]!=0)
    rawrssi[2] = rawrssi[2]*(100-alpha)/100 + analogRead(PIN_RSSI2)*(alpha)/100;
  if (receivermode[3]!=0)
    rawrssi[3] = rawrssi[3]*(100-alpha)/100 + analogRead(PIN_RSSI3)*(alpha)/100;



  for (int i=0;i<=3;i++){

    if (rawrssi[i]<rssimin[i]){ //cutoff minimum
      cutoff_rawrssi[i]=rssimin[i];
    }
    else if (rawrssi[i]>rssimax[i]){ //cutoff maximum
      cutoff_rawrssi[i]=rssimax[i];
    }
    else{
      cutoff_rawrssi[i]=rawrssi[i];
    }
    if (receivermode[i]==0){
      rssi[i]=0; //rssi 0, if no receiver connected 
    }
    else{
      rssi[i]=map(cutoff_rawrssi[i],rssimin[i],rssimax[i],1,255); //map range from calibration, mapping minimum is 1 for connected receivers
    }
    if (rawrssi[i]<=RSSI_DISCONNECTED){
      rssi[i]=0; //if rssi is very low receiver is probably disconnected -> set rssi to 0
    }
  }

}

void timerIsr() //Timer Interrupt, frequency is freq
{
  flag_timerIsr=1; //set flag, timerIsrcall() will be called from loop
}
void timerIsrcall() //calles from loop
{


  calcsyncs(); //reset synccounter and calculate syncs per second

  flag_checkSignals=1; //set flag for calculating signals and select receiver


  if (serial_mode>0 && setupmenu==0) //if csv output activated (and menu closed)
  {

    if (serial_mode==1){ //writing column titles
      serialprint="time;rssi0;rssi1;rssi2;rssi3;rawrssi0;rawrssi1;rawrssi2;rawrssi3;sync0;sync1;sync2;sync3;rating0;rating1;rating2;rating3;selected"; 
      Serial.println(serialprint);
      if (osdconnected) delay(1000);
      serial_mode=2; //switch to data writing
    }

    if (serial_mode==2 && csvsendintervalcounter<=0){ //writing column titles

      if (!osdconnected)
      {
        digitalWrite( PIN_STATUSLED, 1-digitalRead(PIN_STATUSLED));
        serialprint=""; 
        //calculated rssi
        serialprint.concat(millis()); 
        serialprint.concat(";");
        serialprint.concat(rssi[0]); 
        serialprint.concat(";");
        serialprint.concat(rssi[1]); 
        serialprint.concat(";");
        serialprint.concat(rssi[2]); 
        serialprint.concat(";"); 
        serialprint.concat(rssi[3]); 
        serialprint.concat(";"); 
        Serial.print(serialprint);
        //raw analog rssi values
        serialprint="";
        serialprint.concat(rawrssi[0]); 
        serialprint.concat(";");
        serialprint.concat(rawrssi[1]); 
        serialprint.concat(";");
        serialprint.concat(rawrssi[2]); 
        serialprint.concat(";"); 
        serialprint.concat(rawrssi[3]); 
        serialprint.concat(";");
        Serial.print(serialprint);
        //syncs per second
        serialprint="";
        serialprint.concat(sync[0]); 
        serialprint.concat(";");
        serialprint.concat(sync[1]); 
        serialprint.concat(";");
        serialprint.concat(sync[2]); 
        serialprint.concat(";"); 
        serialprint.concat(sync[3]); 
        serialprint.concat(";");
        Serial.print(serialprint);
        //raw analog rssi values
        serialprint="";
        serialprint.concat(syncrating[0]); 
        serialprint.concat(";");
        serialprint.concat(syncrating[1]); 
        serialprint.concat(";");
        serialprint.concat(syncrating[2]); 
        serialprint.concat(";"); 
        serialprint.concat(syncrating[3]); 
        serialprint.concat(";"); 
        Serial.print(serialprint);
        //selected Receiver
        serialprint="";
        serialprint.concat(selectedReceiver);
        Serial.println(serialprint);
      }
      else{ //OSD Connected

        String outputstring="";
        outputstring.concat("  ");
        for (int i=0;i<4;i++){
          if (selectedReceiver==i && receivermode[i]!=0){ //selected receiver and not deac. (if nothings connected)
            //if this receiver is selected
            if (lastselectedby==0){ //receiver manually selected
              outputstring.concat("M");  
            }else if (lastselectedby==1){ //receiver selected by syncs
              outputstring.concat("#");  
            }else if (lastselectedby==2){ //receiver selected by rssi
              outputstring.concat("~");  
            }
          }
          if (receivermode[i]!=0) //if receiver is connected/enabled/active/whatever
            outputstring.concat(rssi[i]); //rssi
          else
            outputstring.concat(" "); //do not display

          if (i<3){
            if (receivermode[i]!=0) //show spacing only if next receiver is connected
              outputstring.concat(" | "); //spacing
          }
        }

        //Center text
        if (outputstring.length()<=25){ //if fits in one line, it should!
          uint8_t prespace=(25-outputstring.length())/2;
          String spacecake="";
          for(int sk=0;sk<prespace;sk++){
            spacecake.concat(" "); //fill with prespace spaces
          }
          outputstring=spacecake+outputstring; //put spaces at begin to center text
        }

        Serial.println(outputstring);

      }

    }


    if (serial_mode==10){ //debug output
      serialprint="debug, rssi: "; 
      serialprint.concat(rssi[0]); 
      serialprint.concat(", ");
      serialprint.concat(rssi[1]); 
      serialprint.concat(", ");
      serialprint.concat(rssi[2]); 
      serialprint.concat(", "); 
      serialprint.concat(rssi[3]);
      Serial.println(serialprint);
    }

  }
  
  
  
  //Serial print menu
  if(setupmenu>0){ //menu opened
    serialprint="";
    if(menu_selected==0){ //entry menu
      if(setupmenu==1){ 
         serialprint.concat("1|RSSI smoothing"); //rssiupdate_alpha
         Serial.println(serialprint);
      }else if(setupmenu==2){ 
         serialprint.concat("2|Sync selection"); //syncselectiondiff
         Serial.println(serialprint);
      }else if(setupmenu==3){ 
         serialprint.concat("3|Switching deadtime"); //switchdeadtime
         Serial.println(serialprint);
      }else if(setupmenu==4){ 
         serialprint.concat("4|Load Defaults"); //load defaults
         Serial.println(serialprint);
      }else if(setupmenu==5){ 
         serialprint.concat("5|Save"); //save to eeprom
         Serial.println(serialprint);
      }else if(setupmenu==6){ 
         serialprint.concat("5|Debug"); //save to eeprom
         Serial.println(serialprint);
      }
      
    }else if(menu_selected==1){ //entry selected
      if(setupmenu==1){ 
         serialprint="1|alpha=";
         serialprint.concat(rssiupdate_alpha);
         Serial.println(serialprint);
      }else if(setupmenu==2){ 
         serialprint="2|syncs=";
         serialprint.concat(syncselectiondiff);
         Serial.println(serialprint);
      }else if(setupmenu==3){ 
         serialprint="3|deadtime=";
         serialprint.concat(switchdeadtime);
         Serial.println(serialprint);
      }else if(setupmenu==4){ 
         serialprint="4|Load default settings, Sure?";
         Serial.println(serialprint);
      }else if(setupmenu==5){ 
         serialprint="5|Save settings to EEPROM, Sure?";
         Serial.println(serialprint);
      }else if(setupmenu==6){ 
         serialprint="5|Debug: ";
         if (menu_debugentry==0){
           serialprint.concat("RawRSSI=");
           serialprint.concat(rawrssi[0]);serialprint.concat(",");
           serialprint.concat(rawrssi[1]);serialprint.concat(",");
           serialprint.concat(rawrssi[2]);serialprint.concat(",");
           serialprint.concat(rawrssi[3]);
         }else if (menu_debugentry==1){
           serialprint.concat("RSSImin=");
           serialprint.concat(rssimin[0]);serialprint.concat(",");
           serialprint.concat(rssimin[1]);serialprint.concat(",");
           serialprint.concat(rssimin[2]);serialprint.concat(",");
           serialprint.concat(rssimin[3]);
         }else if (menu_debugentry==2){
           serialprint.concat("RSSImax=");
           serialprint.concat(rssimax[0]);serialprint.concat(",");
           serialprint.concat(rssimax[1]);serialprint.concat(",");
           serialprint.concat(rssimax[2]);serialprint.concat(",");
           serialprint.concat(rssimax[3]);
         }else if (menu_debugentry==3){
           serialprint.concat("Syncs/sec=");
           serialprint.concat(sync[0]);serialprint.concat(",");
           serialprint.concat(sync[1]);serialprint.concat(",");
           serialprint.concat(sync[2]);serialprint.concat(",");
           serialprint.concat(sync[3]);
         }else if (menu_debugentry==4){
           serialprint.concat("selectedReceiver=");
           serialprint.concat(selectedReceiver);
         }else if (menu_debugentry==5){  //if entry number changed, change also number under "HOLD B"
           serialprint.concat("Receiver 0 mode=");
           serialprint.concat(receivermode[0]);serialprint.concat(" hold to change");
         }else if (menu_debugentry==6){ 
           serialprint.concat("Receiver 1 mode=");
           serialprint.concat(receivermode[1]);serialprint.concat(" hold to change");
         }else if (menu_debugentry==7){ 
           serialprint.concat("Receiver 2 mode=");
           serialprint.concat(receivermode[2]);serialprint.concat(" hold to change");
         }else if (menu_debugentry==8){ 
           serialprint.concat("Receiver 3 mode=");
           serialprint.concat(receivermode[3]);serialprint.concat(" hold to change");
         }
         Serial.println(serialprint);
      }
    }
  }
  
  

  if (csvsendintervalcounter<=0){
    csvsendintervalcounter=CSVSENDINTERVAL; //reset intervalcounter
  }

  csvsendintervalcounter--;
}

void interrupt_sync() //Sync 0 interrupt
{
  interrupt_sync_call();
}

void interrupt_sync_call() //called from mainloop if interrupt_sync flag was set
{
  if (rawsync[interrupt_sync_input]<maxrawsynccounting)
    rawsync[interrupt_sync_input]++; //count up
}

void calcsyncs() //calculate syncs per second
{ 
  sync[interrupt_sync_input]=rawsync[interrupt_sync_input]*freq;
  rawsync[interrupt_sync_input]=0; //reset synccounter

  selectNextSync();
}

void compareAndSelect() //this method compares the values (rssi, syncs) and selects the best signal
{

  uint8_t temp_bestsyncrating=0;
  //uint8_t temp_secondbestsyncrating=0;

  for(int i=1;i<MAXRX;i++){
    if (syncrating[i]>syncrating[temp_bestsyncrating])
    {
      //temp_secondbestsyncrating=temp_bestsyncrating; //save second best
      temp_bestsyncrating=i; //get best sync rating
    }

  }

  /*
 uint16_t syncrating_bestsyncrating=0;
   if (syncrating[temp_bestsyncrating]>0) //abs
   syncrating_bestsyncrating=syncrating[temp_bestsyncrating];
   else
   syncrating_bestsyncrating=-1*syncrating[temp_bestsyncrating];
   
   uint16_t syncrating_selected=0;
   if (syncrating[selectedReceiver]>0) //abs
   syncrating_selected=syncrating[selectedReceiver];
   else
   syncrating_selected=-1*syncrating[selectedReceiver];*/

  uint16_t syncrating_bestsyncrating=syncrating[temp_bestsyncrating]; //per definition >0 and the greatest value
  uint16_t syncrating_selected=syncrating[selectedReceiver]; //lower (or equal) than syncrating_bestsyncrating, can also be negative

  if (syncrating_bestsyncrating-syncrating_selected>=syncselectiondiff || receivermode[selectedReceiver]==2 || receivermode[temp_bestsyncrating]==2 ){ //difference is large enough or one of the receivers has no reliable rssi
    if (syncrating[selectedReceiver]>=syncrating[temp_bestsyncrating]){ //compare and select only by syncs
      selectReceiver(selectedReceiver);
      lastselectedby=1; //selected by syncs
    }else{
      selectReceiver(temp_bestsyncrating);
      lastselectedby=1; //selected by syncs
    }
  }
  else{ //if receivers have rssi connected, and syncs are nearly equal (see variable syncselectiondiff)
    if (rssi[selectedReceiver]>=rssi[temp_bestsyncrating]){ //compare and select by rssi
      selectReceiver(selectedReceiver);
      lastselectedby=2; //selected by rssi
    }else{
      selectReceiver(temp_bestsyncrating);
      lastselectedby=2; //selected by rssi
    }
  }


}
/*void compareAndSelect() //this method compares the values (rssi, syncs) and selects the best signal
 {
 
 uint8_t temp_best=255; //if 255, current selected is the best
 
 for(int i=0;i<4;i++){
 if (i!=selectedReceiver && receivermode[i]!=0){ //skip selectedReceiver and deactivated receivers
 if (signalrating[i]>signalrating[selectedReceiver])
 {
 temp_best=i;
 }
 }
 }
 
 if (temp_best<4)
 selectReceiver(temp_best);
 }*/

void selectReceiver(int sel) //set outputs for demuxer, sel=[0,3]
{
  boolean receiver_changed=false;
  if (sel!=selectedReceiver)
    receiver_changed=true;

  lastswitched=millis(); //set time when last switched

  if (serialPrintOnReceiverChange && selectedReceiver!=sel){ //if enabled and receiver has changed
    csvsendintervalcounter=0; //serial send values immediately
  }

  selectedReceiver=sel;
  if (sel%2==1){
    digitalWrite(PIN_SELECT0,HIGH);
  }
  else{
    digitalWrite(PIN_SELECT0,LOW);
  }
  if (sel/2==1){
    digitalWrite(PIN_SELECT1,HIGH);
  }
  else{
    digitalWrite(PIN_SELECT1,LOW);
  }

  if (receiver_changed && calibration_mode!=2){ //play sound on receiver change and if not in calib mode
    int spkr_freq=440;
    int spkr_duration=0;

    if (sel==0){
      spkr_freq=SNDF_R0;
      spkr_duration=SNDD_R0;
    }
    else if(sel==1){
      spkr_freq=SNDF_R1;
      spkr_duration=SNDD_R1;
    }
    else if(sel==2){
      spkr_freq=SNDF_R2;
      spkr_duration=SNDD_R2;
    }
    else if(sel==3){
      spkr_freq=SNDF_R3;
      spkr_duration=SNDD_R3;
    }

    if (spkr_duration!=0 && setupmenu==0) //no switchsounds in menu
      tone(PIN_SPEAKER,spkr_freq,spkr_duration); //recieverchangesound
  }
}

void selectSync(int sel) //set outputs for muxer sync input, sel=[0,3]
{
  interrupt_sync_input=sel;
  if (sel%2==1){
    digitalWrite(PIN_SELECTSYNC0,HIGH);
  }
  else{
    digitalWrite(PIN_SELECTSYNC0,LOW);
  }
  if (sel/2==1){
    digitalWrite(PIN_SELECTSYNC1,HIGH);
  }
  else{
    digitalWrite(PIN_SELECTSYNC1,LOW);
  }
}
void selectNextSync()
{
  if (interrupt_sync_input>=3){
    interrupt_sync_input=0;
  }
  else{
    interrupt_sync_input++; //switch to next sync
  }

  selectSync(interrupt_sync_input); //set output pins
}

void calibration() //calibration, if calib. done this method should do nothing
{
  if (calibration_mode==1){ //starting calib. mode
    if (digitalRead(PIN_BUTTON_A)==HIGH){ //if button pressed on start
      delay(100);

      Serial.println("Calibration: turn now on transmitter near receivers and press button again");
      calibrationtime_start=millis();

      snd_entercalibmode();

      calibration_mode=2; //calibrate
      digitalWrite( PIN_STATUSLED, HIGH ); //show busy
    }
    else{
      calibration_mode=4; //load calibration
    }
  }
  else if (calibration_mode==2 && (millis()>=calibrationtime_start+CALIBSTART) ){ //calib.mode active

    for (int i=0;i<=3;i++){
      if (rawrssi[i]<rssimin[i]){
        rssimin[i]=rawrssi[i];
        serialprint="\\ ";
        serialprint.concat(i);
        serialprint.concat(":");
        serialprint.concat(rssimin[i]);
        Serial.println(serialprint);
      }
      if (rawrssi[i]>rssimax[i]){
        rssimax[i]=rawrssi[i];
        serialprint="/ ";
        serialprint.concat(i);
        serialprint.concat(":");
        serialprint.concat(rssimax[i]);
        Serial.println(serialprint);
      }
    }


    if (millis()-calibrationtime_start>=CALIBRATIONTIME && (digitalRead(PIN_BUTTON_A)==HIGH) ){ //calibration finished (minimum time and button pressed)
      for (int i=0;i<4;i++){
        if (rssimax[i]<=DEACTIVATERSSI){ //if rssi and syncs are too low 
          //deleted  "&& sync[i]<=DEACTIVATESYNC" (20150318)
          receivermode[i]=0; //deactive receiver input
        }
      }
      print_calibration(); //serial debug
      calibrationtime_start=millis(); //reset starttime, end delay for button release
      calibration_mode =3; //calibration finished, save to eeprom
      digitalWrite( PIN_STATUSLED, LOW);
    }

  }
  else if(calibration_mode==3){ //save to eeprom
    Serial.println("Calibration: saving to EEPROM");
    if (!osdconnected) delay(2000);
    uint8_t addr=0;
    
    uint8_t checkbyte=EEPROM.read(0); //load checkbyte before writing new one //0
    
    EEPROM.write(addr++, EEPROMVERSION); //checkbyte
    EEPROMWriteInt(addr+=2, rssimin[0]);
    EEPROMWriteInt(addr+=2, rssimin[1]);
    EEPROMWriteInt(addr+=2, rssimin[2]);
    EEPROMWriteInt(addr+=2, rssimin[3]);
    EEPROMWriteInt(addr+=2, rssimax[0]);
    EEPROMWriteInt(addr+=2, rssimax[1]);
    EEPROMWriteInt(addr+=2, rssimax[2]);
    EEPROMWriteInt(addr+=2, rssimax[3]);
    EEPROM.write(addr++, receivermode[0]);
    EEPROM.write(addr++, receivermode[1]);
    EEPROM.write(addr++, receivermode[2]);
    EEPROM.write(addr++, receivermode[3]);
    
    if (checkbyte!=EEPROMVERSION){ //if checkbyte was incorrect before writing calibration!
      writeEEPROMsettings(); //write default diversity settings
    }

    snd_calibsaved();

    calibration_mode=5;


  }
  else if(calibration_mode==4){ //load from eeprom
    if (!osdconnected) Serial.println("Calibration: loading from EEPROM");
    uint8_t addr=0;
    uint8_t checkbyte=EEPROM.read(addr++); //checkbyte //0
    if (checkbyte==EEPROMVERSION){ //if checkbyte correct
      rssimin[0]=EEPROMReadInt(addr+=2); //1
      rssimin[1]=EEPROMReadInt(addr+=2); //3
      rssimin[2]=EEPROMReadInt(addr+=2); //5
      rssimin[3]=EEPROMReadInt(addr+=2); //7
      rssimax[0]=EEPROMReadInt(addr+=2); //9
      rssimax[1]=EEPROMReadInt(addr+=2); //11
      rssimax[2]=EEPROMReadInt(addr+=2); //13
      rssimax[3]=EEPROMReadInt(addr+=2); //15
      receivermode[0]=EEPROM.read(addr++); //17
      receivermode[1]=EEPROM.read(addr++); //18
      receivermode[2]=EEPROM.read(addr++); //19
      receivermode[3]=EEPROM.read(addr++); //20
      //if addresses changed, other eeprom load/write addresses need to be changed, search "eepromsettingsaddress"
      

      print_calibration(); //serial debug

      snd_startup();

      calibration_mode=6;  //load other settings, then normal mode 0
    }
    else{
      Serial.println("  checkbyte incorrect. Calibrate!");
      snd_calibloadfailed();
      if (osdconnected)  delay(8000); //osd delay
      calibration_mode=0; //normal mode, load nothing else
    }

  }
  else if(calibration_mode==5){ //wait
    digitalWrite( PIN_STATUSLED, HIGH);
    if (millis()-calibrationtime_start>=2000){ //wait, button release
      calibration_mode=6; //load other settings, then normal mode 0
      digitalWrite( PIN_STATUSLED, LOW);
    }
  }
  else if(calibration_mode==6){ //load other settings
    loadEEPROMsettings();
    calibration_mode=0; //normal operation mode
  }
  else if(calibration_mode==0){ //normal operation mode, !! Executes from loop !!
    if (serial_mode==0){ //start serial data output for osd or log
      serial_mode=1; //starting csv output
    } 
  }

}


void loadEEPROMsettings() //load diversity settings
{
  uint8_t addr=0;
  uint8_t checkbyte=EEPROM.read(addr++); //checkbyte //0
  if (checkbyte==EEPROMVERSION){ //if checkbyte correct
    addr=21; //eepromsettingsaddress
    rssiupdate_alpha=EEPROM.read(addr++); //uint8_t  //21
    syncselectiondiff=EEPROMReadInt(addr+=2); //uint16_t //22
    switchdeadtime=EEPROMReadInt(addr+=2); //uint16_t //24
    receivermode[0]==EEPROM.read(addr++);
    receivermode[1]==EEPROM.read(addr++);
    receivermode[2]==EEPROM.read(addr++);
    receivermode[3]==EEPROM.read(addr++);
     
  }
  
}

void writeEEPROMsettings() //save diversity settings
{
  uint8_t addr=0;
  uint8_t checkbyte=EEPROM.read(addr++); //checkbyte //0
  //if (checkbyte==EEPROMVERSION){ //if checkbyte correct
    addr=21; //eepromsettingsaddress
    EEPROM.write(addr++, rssiupdate_alpha); //uint8_t //21 
    EEPROMWriteInt(addr+=2, syncselectiondiff); //22
    EEPROMWriteInt(addr+=2, switchdeadtime); //24
    EEPROM.write(addr++, receivermode[0]); //26
    EEPROM.write(addr++, receivermode[1]); //27
    EEPROM.write(addr++, receivermode[2]); //28
    EEPROM.write(addr++, receivermode[3]); //29
     
  //}
  
}

void loadDiversitySettingDefaults(){
  rssiupdate_alpha=DEFAULT_RSSIUPDATE_ALPHA;
  syncselectiondiff=DEFAULT_SYNCSELECTIONDIFF;
  switchdeadtime=DEFAULT_SWITCHDEADTIME;
}


void calcSignalRating()
{
  uint16_t temp_syncfails[MAXRX]={
    0,0,0,0  }; 

  for (int i=0;i<MAXRX;i++){ //for all receivers
    if (receivermode[i]!=0) //receiver is activated
    {

      if (sync[i]>16000){//only the deviation from 16k
        temp_syncfails[i]=sync[i]-16000;
        if (temp_syncfails[i]>MAXSYNCFAILS) //cutoff, so that blackscreen(not conncted) is not better than static
          temp_syncfails[i]=MAXSYNCFAILS;
      }
      else{
        temp_syncfails[i]=16000-sync[i];
      }

    }
    else{ //receiver i is deactivated/not connected
    }
  }

  for (int i=0;i<MAXRX;i++){ //for all receivers
    if (receivermode[i]!=0) //receiver is activated
    {
      uint16_t lowestothersyncfails=65535;

      for (int j=0;j<MAXRX;j++){ //compare every receiver with every other
        if (j!=i && receivermode[j]!=0) //receiver is activated and not i
        {

          if (lowestothersyncfails>temp_syncfails[j])
            lowestothersyncfails=temp_syncfails[j];

        }
      }

      syncrating[i]=lowestothersyncfails/2-temp_syncfails[i]/2; //>0 if sync from i is better compared to the others

    }
  }

  /* //check for signal loss
   boolean signalok=false;
   for (int i=0;i<MAXRX;i++){
   if (receivermode[i]!=0 && rssi[i]>=SIGNALLOSSRSSI) //rssi from activated receiver is ok
   signalok=true; //signal is ok
   if (receivermode[i]!=0 && rssi[i]>=SIGNALGOODRSSI) //rssi from activated receiver is good
   snd_signallost=0; //reset sound, so it can play again
   }
   if (signalok==false){ //signal is probably lost
   if (snd_signallost==0){ //sound not played
   tone(PIN_SPEAKER,SNDF_SIGNALLOST,SNDD_SIGNALLOST);
   snd_signallost=1; //sound played
   }
   }*/

}
/*void calcSignalRating() //alte version
 {
 for (int i=0;i<4;i++){ //for all receivers
 if (receivermode[i]!=0) //receiver is activated
 {
 uint16_t temp_syncfails=0;
 if (sync[i]>16000){
 temp_syncfails=sync[i]-16000;
 }else{
 temp_syncfails=16000-sync[i];
 }
 //Function to calculate signalrating from missing syncs and rssi
 uint16_t syncquality= 255-(temp_syncfails)/256; //[0,255]
 signalrating[i]=rssi[i]+syncquality;
 
 }else{ //receiver i is deactivated/not connected
 signalrating[i]=0;
 }
 
 }
 }*/



void nextReceiver_manual(){ //manually switch to next receiver
  uint8_t temp_csr=selectedReceiver;
  temp_csr++;
  if (temp_csr>=MAXRX)
    temp_csr=0;

  while(receivermode[temp_csr]==0 ){ //cycle through receivers until one is not disabled
    temp_csr++;
    if (temp_csr>=MAXRX)
      temp_csr=0;
  }
  
  lastselectedby=0; //manually selected receiver
  selectReceiver(temp_csr); //switch this receiver 
}

void print_calibration()
{
  //prints calibration. [From]+|-[To],[From]+|-[To],...  where - between from and to means disabled, + means enabled
  serialprint="calib: ";  
  Serial.print(serialprint); 
  serialprint=""; 
  serialprint.concat(rssimin[0]); 
  if (receivermode[0]!=0) serialprint.concat("+"); 
  else serialprint.concat("-");
  serialprint.concat(rssimax[0]); 
  serialprint.concat(",");
  Serial.print(serialprint);
  serialprint=""; 
  serialprint.concat(rssimin[1]); 
  if (receivermode[1]!=0) serialprint.concat("+"); 
  else serialprint.concat("-");
  serialprint.concat(rssimax[1]); 
  serialprint.concat(",");
  Serial.print(serialprint);
  serialprint=""; 
  serialprint.concat(rssimin[2]); 
  if (receivermode[2]!=0) serialprint.concat("+"); 
  else serialprint.concat("-");
  serialprint.concat(rssimax[2]); 
  serialprint.concat(",");
  Serial.print(serialprint);
  serialprint=""; 
  serialprint.concat(rssimin[3]); 
  if (receivermode[3]!=0) serialprint.concat("+"); 
  else serialprint.concat("-");
  serialprint.concat(rssimax[3]);
  Serial.println(serialprint);

  Serial.flush();

  /*serialprint=""; Moved to print in a single line, see above
   for (int i=0;i<4;i++) {
   serialprint.concat("Input ");
   serialprint.concat(i);
   serialprint.concat(" ");
   if (receivermode[i]==0){
   serialprint.concat("deac. ;");
   }else if (receivermode[i]==1){
   serialprint.concat("acti. ;");
   }
   }
   Serial.println(serialprint);*/
}

void EEPROMWriteInt(uint8_t paddr,uint16_t pdata){
  EEPROM.write(paddr, (uint8_t)(pdata%256));
  EEPROM.write(++paddr, (uint8_t)(pdata/256));
}

uint16_t EEPROMReadInt(uint8_t paddr){
  uint16_t data=EEPROM.read(paddr);
  data+=EEPROM.read(++paddr)*256;
  return data;
}

uint16_t abs_int32(int32_t p){
  if (p>0){
    return (uint16_t)p;
  }
  else{
    return (uint16_t)(p*-1);
  }
}


void snd_startup(){
  tone(PIN_SPEAKER,SNDF_CALIBLOADED,SNDD_CALIBLOADED); //Startup sound
  delay(SNDD_CALIBLOADED+100); //wait until last sound finished
}
void snd_entereddiversitymode(){
  snd_startup();
}
void snd_enteredmanualmode(){
  tone(PIN_SPEAKER,SNDF_MANUALMODE,SNDD_MANUALMODE);
  delay(SNDDELAY_MANUALMODE);
  tone(PIN_SPEAKER,SNDF_MANUALMODE2,SNDD_MANUALMODE2);
  delay(SNDD_MANUALMODE2+100); //wait until last sound finished
}


void snd_entercalibmode(){
  tone(PIN_SPEAKER,SNDF_CALIBRATION,SNDD_CALIBRATION); //enter calibration mode sound
}
void snd_calibloadfailed(){
  tone(PIN_SPEAKER,SNDF_CALIBLOADFAILED_1,SNDD_CALIBLOADFAILED_1);
  delay(SNDDELAY_CALIBLOADFAILED); //sound delay
  tone(PIN_SPEAKER,SNDF_CALIBLOADFAILED_2,SNDD_CALIBLOADFAILED_2);     
}
void snd_calibsaved(){
  tone(PIN_SPEAKER,SNDF_CALIBSAVED,SNDD_CALIBSAVED);
}

void snd_loadeddefaults(){
  snd_startup();
}
void snd_eepromwritten(){
  snd_calibsaved();
}

void snd_menuup(){
  tone(PIN_SPEAKER,SNDF_MENUUP,SNDD_MENUUP); //enter calibration mode sound
}
void snd_menudown(){
  tone(PIN_SPEAKER,SNDF_MENUDOWN,SNDD_MENUDOWN); //enter calibration mode sound
}
void snd_menuopen(){
  tone(PIN_SPEAKER,SNDF_MENUOPEN,SNDD_MENUOPEN); //enter calibration mode sound
}
void snd_menuclosed(){
  if (mode==0){ //diversitymode
    snd_entereddiversitymode();
  }else if(mode==1){ //manualmode
    snd_enteredmanualmode(); 
  }
}
void snd_menuselected(){
  snd_menuopen();
}
void snd_menudeselected(){
  snd_menuclosed();
}
void snd_menuselectedborder(){
  delay(SNDD_MENUDOWN+10); //wait for menuup/down sound to finish
  tone(PIN_SPEAKER,SNDF_MENUSELECTEDBORDER,SNDD_MENUSELECTEDBORDER); //enter calibration mode sound
}


